# -*- encoding: utf-8 -*-
'''clslq_singleton

Created: 2021/08/23 14:27:30

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''


def clslq_singleton(cls, *args, **kv):
    """Wrapper function to construct a singleton, usage:

    @clslq_singleton

    class Cls(object):
        def __init__(self):
            pass

    :return: a singleton class
    :rtype: object
    """
    _instance = {}

    def inner():
        if cls not in _instance:
            _instance[cls] = cls(*args, **kv)
        return _instance[cls]

    return inner


class SingletonClass(object):
    """ Singleton class wapper
    Only support **__init__** function without parameters, usage:

    class Cls(SingletonClass):
        def __init__(self):
            pass

    """
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(SingletonClass, cls)
            cls._instance = orig.__new__(cls, *args, **kw)
        return cls._instance


class SingletonMetaclass(type):
    """Metaclass implement, usage:

    class Cls(metaclass=SingletonMetaclass):
        pass
    """
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonMetaclass,
                                        cls).__call__(*args, **kwargs)
        return cls._instances[cls]


"""
Some other singleton class implement method
#--------------------------------------------------------------------
# Use __new__
#--------------------------------------------------------------------
class Single(object):
    _instance = None
    def __new__(cls, *args, **kw):
        if cls._instance is None:
            cls._instance = object.__new__(cls, *args, **kw)
        return cls._instance
    def __init__(self):
        pass
"""



# -*- encoding: utf-8 -*-
'''clslq_md5

Created: 2021/08/23 14:27:12

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''


import hashlib


class ClslqMd5(object):
    """wheel of md5
    """
    def __init__(self) -> None:
        super().__init__()

    def __repr__(self) -> str:
        return super().__repr__()

    def file(self, filename):
        '''
        :param filename: calc md5 digest from a file
        :return: md5 string
        '''
        m = hashlib.md5()
        with open(filename, 'rb') as f:
            while True:
                data = f.read(4096)
                if not data:
                    break
                m.update(data)
        return m.hexdigest()

    def string(self, content):
        '''
        :param content: calc digest from string
        :return: md5 string
        '''
        m = hashlib.md5(content)
        return m.hexdigest()

    def same(self, f1, f2):
        '''
        :param f1: first filename or content<string>
        :param f2: second filename or content<string>
        :return: md5(f1) == md5(f2)
        '''
        if type(f1) == type(str) == type(f2):
            md51 = self.string(f1)
            md52 = self.string(f2)
        else:
            md51 = self.file(f1)
            md52 = self.file(f2)
        return md51 == md52


# -*- encoding: utf-8 -*-
'''cli_clslq_pip

Usage: clslq pip [OPTIONS]

  The wrapper for pip, use local pypi as default.

Options:
  -t, --trusted-host TEXT  The trusted mirror host, default:
                           gw.lovelacelee.com.
  -i, --pypi TEXT          The pypi mirror url, default use:
                           https://pypi.lovelacelee.com/
  -h, --help               Show this message and exit.

'''



import click
import platform
import os

from .clslq_utils import pipguess


@click.option(
    '--pypi',
    '-i',
    default='https://pypi.lovelacelee.com/',
    help='The pypi mirror url, default use: https://pypi.lovelacelee.com/')
@click.option('--trusted-host',
              '-t',
              default='gw.lovelacelee.com',
              help='The trusted mirror host, default: gw.lovelacelee.com.')
@click.command(context_settings=dict(
    allow_extra_args=True,
    ignore_unknown_options=True,
),
               help="The wrapper for pip, use local pypi as default.")
@click.pass_context
def pip(ctx, pypi, trusted_host):
    """Wrapper of pip

    Supported commands:
    ```
    # upgrade pip first
    # python -m pip uninstall pip -y
    # python -m ensurepip
    # python -m pip install -U pip
    # os.system(pipguess()+'install --upgrade pip')
    ```

    Args:
        pypi (<str>): Available pypi mirror url
        trusted_host (<str>): pypi mirror domain
    """
    #click.echo(ctx.args)
    _cmdline = pipguess()
    _change_pypi_cmds = ['install', 'download', 'list', 'search']
    _change_pypi = False
    for i in ctx.args:
        _cmdline += ' ' + i + ' '
        if i in _change_pypi_cmds:
            _change_pypi = True
    if _change_pypi:
        _cmdline += ' -i ' + pypi
        _cmdline += ' --trusted-host ' + trusted_host
    click.echo(_cmdline)
    click.echo('=W=H=A=T=R=E=T=U=R=N=E=D=B=Y=P=I=P=')
    os.system(_cmdline)



# -*- encoding: utf-8 -*-
'''clslq_utils

Created: 2021/08/23 14:27:43

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''



import os
import sys
import platform
import re
import shutil
import click
from .clslq_md5 import ClslqMd5


def mkdir_p(absolute_path):
    """mkdir -p implement
    
    Usage: 
    
    mkdir_p('D:\\A\\B\\C.txt')

    mkdir_p('~/A/B/C')
    """
    path = os.path.dirname(absolute_path)
    if not os.path.exists(path):
        os.makedirs(path, 0o777)


def dir_copy(srcpath, dstpath):
    try:
        if not os.path.exists(srcpath):
            return
        if not os.path.exists(dstpath):
            if sys.version_info.major >= 3 and sys.version_info.minor >= 8:
                shutil.copytree(srcpath, dstpath, dirs_exist_ok=True)
            else:
                shutil.copytree(srcpath, dstpath)
    except Exception as e:
        pass


def rmdir(path):
    """
    Warning: all files and directories in path will be deleted.
    """
    if os.path.isfile(path):
        print("{} will be deleted.".format(path))
        os.remove(path)
        return
    for root, dirs, files in os.walk(path):
        for d in dirs:
            t = os.path.join(root, d)
            if os.path.exists(t):
                print("{} will be deleted.".format(t))
                shutil.rmtree(t)
        for f in files:
            t = os.path.join(root, f)
            if os.path.exists(path):
                print("{} will be deleted.".format(t))
                os.remove(t)


def win_runtime_cp(src, to):
    # useful while running a package by pyinstaller on windows
    if platform.system() == "Windows":
        for path in sys.path:
            if re.match(r'^_MEI\d+$', os.path.basename(path)):
                if os.path.exists(path):
                    dir_copy(src, to)
                    break


def is_frozen():
    # All of the modules are built-in to the interpreter, e.g., by py2exe
    return hasattr(sys, "frozen")


def pipguess():
    if platform.system() == "Windows":
        return "python -m pip "
    else:
        return "python3 -m pip "


def setenv(permanent=True, key=None, value=None):
    if permanent:
        """
        HERE is the way for permanently env-set under windows
        Administrator is required
        # with /m means system env
        # without /m means user env
        """
        systype = platform.system()
        if systype == "Windows":
            os.system(r"setx %s %s /m" % (key, value))
        else:
            os.environ['%s' % key] = value
    else:
        os.environ['%s' % key] = value


def pip_conf_install(src=None):
    try:
        if not src or not os.path.exists(src):
            src = os.path.join(os.path.dirname(__file__), 'pip.conf')
        systype = platform.system()
        if systype == "Windows":
            pipdotdir = os.path.join(os.getenv('APPDATA'), "pip")
            pip_dest = os.path.join(pipdotdir, "pip.ini")
        elif systype == "Darwin":
            pipdotdir = os.path.join(os.getenv('HOME'), "/Library/Application\ Support/pip/pip")
            pip_dest = os.path.join(pipdotdir, "pip.conf")
        else:
            pipdotdir = os.path.join(os.getenv('HOME'), ".pip")
            pip_dest = os.path.join(pipdotdir, "pip.conf")
        mkdir_p(pipdotdir)
        md5 = ClslqMd5()
        if os.path.exists(pip_dest) and md5.same(src, pip_dest):
            click.secho("{} exist already".format(pip_dest))
            return
        shutil.copyfile(src, pip_dest)
    except:
        pass



# -*- encoding: utf-8 -*-
'''
cli

click.group(name='main')

'''
import os
import click
from click.termui import prompt
from clslq.cli_clslq_pip import pip
from clslq.cli_clslq_venv import venv
from clslq.cli_clslq_notion import notion

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(name="main", context_settings=CONTEXT_SETTINGS)
@click.version_option(message='%(prog)s-%(version)s')
def main():
    """
    CLSLQ is a python library and command toolsets of Connard.
    Most of the contents are written in progress of python learning.
    CLSLQ include some quick-start python programming functions, wrappers and tools.
    For more information, please contact [lovelacelee@gmail.com].
    Documents available on https://clslq.readthedocs.io/.
    MIT License Copyright (c) Connard Lee.
    """
    pass


main.add_command(pip, name='pip')
main.add_command(venv, name='venv')
main.add_command(notion, name='notion')

if __name__ == '__main__':
    main()



# -*- encoding: utf-8 -*-
'''cli_clslq_notion

Notion client in python, notion-py is required

Usage: clslq notion [OPTIONS]

[openpyxl](https://openpyxl-chinese-docs.readthedocs.io/zh_CN/latest/tutorial.html)

[pandas](https://www.pypandas.cn/)

Notion template on https://airy-skiff-4d0.notion.site/04143158def3413fb58e7dae4b2d9aff

'''

import click
import datetime
import string
import time
import json
import os
import requests
import base64
import re
import traceback
import pandas
import calendar
# Support email
import smtplib
from io import BytesIO
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.header import Header

from openpyxl import Workbook
from openpyxl.styles import Font
from openpyxl.styles import colors
from openpyxl.styles import Alignment
from openpyxl.styles import Border
from openpyxl.styles import Side
from openpyxl.styles import PatternFill

from .clslq_config import ClslqConfigUnique
from .clslq_log import ClslqLogger
from notion_client import Client

from .templates import weekreport
from .templates import monthreport

clslog = ClslqLogger().log


class Report(object):
    """Base class of Report

    Args:
        object (wbname): Workbook name, generated document file name
    """

    def __init__(self, wbname):
        self.wb = Workbook()
        self.sht = self.wb.active
        self.default_border = Border(left=Side(border_style='thin',
                                               color='000000'),
                                     right=Side(border_style='thin',
                                                color='000000'),
                                     top=Side(border_style='thin',
                                              color='000000'),
                                     bottom=Side(border_style='thin',
                                                 color='000000'))
        self.wbname = wbname
        self.datetime_now = datetime.datetime.now()

    @property
    def now(self):
        return self.datetime_now

    def render_html_without_inline_css(self, title, df):
        """Render html form template, use pandas

        Note that some of the email display methods only support inline-css style,
        This method does not use any inline-css for table headers and rows.

        Deprecated

        Args:
            title (str): Title of html and email subject
            df (object): Pandas DataFrame object
        """
        clslog.info("Render html for {}".format(title))
        with open(self.wbname + '.html', encoding='utf-8', mode='w') as f:

            task = df[df[u'分类'] != u'工作计划']
            plan = df[df[u'分类'] == u'工作计划']

            t = string.Template(weekreport.wr_template)
            f.write(
                t.safe_substitute({
                    "title":
                    title,
                    "table":
                    task.to_html(classes='tablestyle', index=False, na_rep=""),
                    "plan":
                    plan.to_html(classes='tablestyle', index=False, na_rep="")
                }))

    def send_study_email(self, config, title, email_html):
        """Send report email to receivers defined in .clslq.json

        Args:
            config (dict): Loaded json object from .clslq.json
            title (str): Unicode string as email subject
        """
        email = config.get('email')
        smtpserver = email['sender']['smtpserver']
        user = email['sender']['user']
        pwd = email['sender']['pwd']
        receivers = email['study_receivers']
        clslog.info("{} {} {}".format(smtpserver, user, title))

        msg = MIMEMultipart()
        msg['From'] = "{}".format(user)
        msg['To'] = ",".join(receivers)
        msg['Subject'] = Header(title, 'utf-8')
        msg.attach(MIMEText(email_html, 'html', 'utf-8'))
        try:
            smtp = smtplib.SMTP()
            smtp.connect(smtpserver)
            smtp.login(user, pwd)
            smtp.sendmail(user, receivers, msg.as_string())
            clslog.info('Email sent to {} done'.format(receivers))
            smtp.quit()
            smtp.close()
        except Exception as e:
            clslog.critical(
                "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))

    def send_email(self, config, title):
        """Send report email to receivers defined in .clslq.json

        Args:
            config (dict): Loaded json object from .clslq.json
            title (str): Unicode string as email subject
        """
        email = config.get('email')
        smtpserver = email['sender']['smtpserver']
        user = email['sender']['user']
        pwd = email['sender']['pwd']
        receivers = email['receivers']
        clslog.info("{} {} {}".format(smtpserver, user, title))

        msg = MIMEMultipart()
        msg['From'] = "{}".format(user)
        msg['To'] = ",".join(receivers)
        msg['Subject'] = Header(title, 'utf-8')
        with open(self.wbname + '.html', "r", encoding='utf-8') as f:
            msg.attach(MIMEText(f.read(), 'html', 'utf-8'))
        try:
            smtp = smtplib.SMTP()
            smtp.connect(smtpserver)
            smtp.login(user, pwd)
            smtp.sendmail(user, receivers, msg.as_string())
            clslog.info('Email sent to {} done'.format(receivers))
            smtp.quit()
            smtp.close()
        except Exception as e:
            clslog.critical(
                "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))

    def remove_files(self):
        """Removes all generated files"""
        if os.path.exists(self.wbname + '.html'):
            os.remove(self.wbname + '.html')
        if os.path.exists(self.wbname + '.xlsx'):
            os.remove(self.wbname + '.xlsx')


class WeekReport(Report):
    def week_belongs(self, sdate, edate):
        s = sdate - datetime.timedelta(days=1)
        e = edate + datetime.timedelta(days=1)
        clslog.info("[{} {}] {}".format(s, e, self.datetime_now))
        if self.datetime_now >= (sdate - datetime.timedelta(
                days=2)) and self.datetime_now <= (edate +
                                                   datetime.timedelta(days=2)):
            return True
        else:
            return False

    def set_worksheet_head(self, head):
        """Set sheet head

        Args:
            head (worksheet head): string
        """
        self.sht.merge_cells('A1:F1')

        self.sht['A1'] = u"本周工作情况" + head.replace('-', "")
        self.sht['A1'].alignment = Alignment(horizontal='center',
                                             vertical='center')
        self.sht['A1'].font = Font(color=colors.BLACK, b=True, size=14)
        self.sht['A1'].fill = PatternFill("solid", fgColor="00FF8080")
        self.sht.row_dimensions[1].height = 20

    def set_worksheet_title(self):
        u"""openpyxl 不支持按列设置样式
        """
        self.sht['A2'] = u"分类"
        self.sht['B2'] = u"事项"
        self.sht['C2'] = u"进展"
        self.sht['D2'] = u"问题"
        self.sht['E2'] = u"解决"
        self.sht['F2'] = u"评审、复盘、总结"
        for col in range(1, 7):
            self.sht.cell(column=col, row=2).font = Font(name=u'微软雅黑',
                                                         bold=True,
                                                         size=12)

    def content_parse_title(self, item):
        result = None
        try:
            title = item[u'名称']['title']
            for i in title:
                result = i['plain_text']
        except Exception as e:
            pass
        finally:
            return result

    def content_parse_state(self, item, row):
        result = ''
        try:
            for i in item[u'状态']['multi_select']:
                if i['name'] == u'进行中':
                    for col in range(1, 7):
                        self.sht.cell(column=col, row=row).fill = PatternFill(
                            "solid", fgColor="00CCFFCC")
                if i['name'] == u'遇问题':
                    for col in range(1, 7):
                        self.sht.cell(column=col, row=row).fill = PatternFill(
                            "solid", fgColor="00FFCC99")
                result = "{} {}".format(result, i['name'])
        except Exception as e:
            pass
        finally:
            return result

    def content_parse_type(self, item, row):
        result = ''
        try:
            result = item[u'分类']['select']['name']
            if result == u'工作计划':
                self.sht.cell(column=6, row=row).value = u"下周工作计划"
                for col in range(1, 7):
                    self.sht.cell(column=col, row=row).fill = PatternFill(
                        "solid", fgColor="00CCFFCC")
        except Exception as e:
            pass
        finally:
            return result

    def content_parse_richtext(self, item, text):
        """Parse Notion Column content

        Args:
            item (dict): Notion Column content
            text (str): Unicode string means column title

        Returns:
            str: Cell result
        """
        result = ''
        try:
            title = item[text]['rich_text']
            for i in title:
                result = i['plain_text']
        except Exception as e:
            pass
        finally:
            return result.replace('\n', ' ')

    def excel_worksheet_fill(self, item, row):
        self.sht['A' + str(row)] = self.content_parse_type(item, row)
        self.sht['B' + str(row)] = self.content_parse_title(item)
        self.sht['C' + str(row)] = self.content_parse_state(item, row)
        self.sht['D' + str(row)] = self.content_parse_richtext(item, u'问题')
        self.sht['E' + str(row)] = self.content_parse_richtext(item, u'解决方法')
        self.sht['F' + str(row)] = self.content_parse_richtext(
            item, u'评审、复盘、总结')

        for c in ('A', 'C'):
            self.sht.column_dimensions[c].width = 10
            for cell in self.sht[c]:
                cell.alignment = Alignment(horizontal='center',
                                           vertical='center',
                                           wrap_text=True)
                cell.border = self.default_border
        for c in ('B', 'F', 'D', 'E'):
            self.sht.column_dimensions[c].width = 30
            for cell in self.sht[c]:
                cell.alignment = Alignment(horizontal='center',
                                           vertical='center',
                                           wrap_text=True)
                cell.border = self.default_border

    def excel_worksheet_dump(self, wbname, database):
        # Change sheet name
        self.sht.title = 'WR'
        self.set_worksheet_head(wbname)
        self.set_worksheet_title()

        row = 3
        for node in database['results']:
            item = node['properties']
            self.excel_worksheet_fill(item, row)
            row = row + 1
        self.wb.save(filename=wbname + '.xlsx')
        self.wb.close()

    def pandas_df_fill(self, database):
        _type = []
        _title = []
        _state = []
        _problem = []
        _solve = []
        _conclusion = []
        for node in database['results']:
            item = node['properties']
            _type.append(self.content_parse_type(item, 0))
            _title.append(self.content_parse_title(item))
            _state.append(self.content_parse_state(item, 0))
            _problem.append(self.content_parse_richtext(item, u'问题'))
            _solve.append(self.content_parse_richtext(item, u'解决方法'))
            _conclusion.append(self.content_parse_richtext(item, u'评审、复盘、总结'))
        data = {
            u'分类': pandas.Series(_type, index=range(len(_type))),
            u'事项': pandas.Series(_title, index=range(len(_title))),
            u'进展': pandas.Series(_state, index=range(len(_state))),
            u'问题': pandas.Series(_problem, index=range(len(_problem))),
            u'解决': pandas.Series(_solve, index=range(len(_solve))),
            u'评审复盘总结备注': pandas.Series(_conclusion,
                                       index=range(len(_conclusion)))
        }
        self.df = pandas.DataFrame(data)
        return self.df

    def render_html(self, clsconfig, title, database):
        """Render html form template

        Note that some of the email display methods only support inline-css style,
        This method support inline-css for table headers and rows.

        Args:
            title (str): Title of html and email subject
            df (object): Pandas DataFrame object
        """
        clslog.info("Render[Inline-CSS] html for {}".format(title))
        with open(self.wbname + '.html', encoding='utf-8', mode='w') as f:
            table = str('')
            summary = str('')
            plan = str('')
            i = 0
            j = 0

            task_paln_template = """
                <tr height="19" style="height:14.0pt;background: {color}">
                    <td style="border: 0.5pt solid #cfcfcf; vertical-align: middle; text-align: center;">{type}</td>
                    <td style="border: 0.5pt solid #cfcfcf;">{title}</td>
                    <td style="border: 0.5pt solid #cfcfcf; vertical-align: middle; text-align: center;">{state}</td>
                    <td style="border: 0.5pt solid #cfcfcf;">{problem}</td>
                    <td style="border: 0.5pt solid #cfcfcf;">{solve}</td>
                </tr>
            """
            summary_template = """
                <tr>
                    <td style="padding: 10px; background-color: rgba(204, 204, 204, 0.1)">
                    <span style="font-size: 16px; color: #81e4c3">●</span>&nbsp;
                    <span>
                        <span style="border-bottom: 1px dashed rgb(204, 204, 204); position: relative;">{summary}</span>
                    </span>
                    </td>
                </tr>
            """
            for node in database['results']:
                item = node['properties']

                def bgcolor(i):
                    return '#F7F7F7' if i % 2 == 0 else '#fff'

                if self.content_parse_type(item, 0) == u'工作计划':
                    plan += task_paln_template.format(
                        **{
                            'type': str(i + 1),
                            'title': self.content_parse_title(item),
                            'state': "-",
                            'problem': self.content_parse_richtext(
                                item, u'问题'),
                            'solve': self.content_parse_richtext(
                                item, u'解决方法'),
                            'color': bgcolor(i)
                        })
                    i = i + 1
                else:
                    table += task_paln_template.format(
                        **{
                            'type': self.content_parse_type(item, 0),
                            'title': self.content_parse_title(item),
                            'state': self.content_parse_state(item, 0),
                            'problem': self.content_parse_richtext(
                                item, u'问题'),
                            'solve': self.content_parse_richtext(
                                item, u'解决方法'),
                            'color': bgcolor(j)
                        })
                    j = j + 1
                summarize_item = self.content_parse_richtext(item, u'评审、复盘、总结')
                if len(summarize_item):
                    summary += summary_template.format(
                        **{'summary': summarize_item})
            html = string.Template(weekreport.wr_template)

            f.write(
                html.safe_substitute({
                    "title": title,
                    "table": table,
                    "plan": plan,
                    "user": clsconfig.get('user'),
                    "department": clsconfig.get('department'),
                    "summarize": summary
                }))


class MonthReport(Report):
    def belongs(self, date):
        if date > self._mdaystart and date <= self._mdayend:
            return True
        else:
            return False

    def week_belongs(self, sdate, edate):
        if sdate >= (self._mdaystart -
                     datetime.timedelta(days=1)) and edate <= (
                         self._mdayend + datetime.timedelta(days=1)):
            return True
        else:
            return False

    def init(self, title, user, department, force):

        nowdate = self.datetime_now
        if nowdate.month == 1:
            lastmonth = 12
        else:
            lastmonth = nowdate.month - 1
        self._mdaystart = nowdate - \
            datetime.timedelta(
                days=(nowdate.day + calendar.mdays[lastmonth]))
        if force:
            self._mdayend = nowdate
        else:
            self._mdayend = nowdate - datetime.timedelta(days=nowdate.day)

        clslog.critical("==月报检索{}==>{}周报及读书学习笔记==".format(
            self._mdaystart.strftime("%Y%m%d"),
            self._mdayend.strftime("%Y%m%d")))

        # 当月发上月报告
        mtitle = "{}({}{:02})".format(title, nowdate.year, lastmonth)
        click.secho("{}".format(mtitle), fg='blue')

        self._title = mtitle

        self._book = str('')
        self._book_content = str('')
        self._study_list = str('')
        self._main_target = str('')
        self._team_target = str('')
        self._technology = str('')
        self._patent = str('')
        self._review = str('')
        self._tech_issues = str('')
        self._maintainance = str('')
        self._duties = str('')
        self._programming_tasks = str('')
        self._reading_share = str('')
        _template = """
            <li style="list-style-type: upper-roman;">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span></li>
        """
        self._reading_share += _template.format(**{
            'content': u"项目例会中结合技术管理、结构思考等方面书中所学",
            'solve': u"会议提议与解决着重高效"
        })
        self._reading_share += _template.format(
            **{
                'content': u"日常工作结合技术分享精神",
                'solve': u"利用自身高效整理的特点，将团队技术栈相关的有用信息分享给团队开发人员"
            })
        self._directions = str('')

        self._user = user
        self._department = department

    @property
    def mtitle(self):
        return self._title

    @mtitle.setter
    def mtitle(self, xtitle):
        if not isinstance(xtitle, str):
            raise ValueError("mtitle must be an string")
        else:
            self._title = xtitle

    def content_parse_title(self, item):
        result = None
        try:
            title = item[u'名称']['title']
            for i in title:
                result = i['plain_text']
        except Exception as e:
            pass
        finally:
            return result

    def content_parse_state(self, item):
        result = ''
        try:
            for i in item[u'状态']['multi_select']:
                result = "{} {}".format(result, i['name'])
        except Exception as e:
            pass
        finally:
            return result

    def content_parse_type(self, item):
        result = ''
        try:
            result = item[u'分类']['select']['name']
            if result == u'工作计划':
                return ''
        except Exception as e:
            pass
        finally:
            return result

    def content_parse_richtext(self, item, text):
        """Parse Notion Column content

        Args:
            item (dict): Notion Column content
            text (str): Unicode string means column title

        Returns:
            str: Cell result
        """
        result = ''
        try:
            title = item[text]['rich_text']
            for i in title:
                result = i['plain_text']
        except Exception as e:
            pass
        finally:
            return result.replace('\n', ' ')

    def block_parse_table_cell_properties(self, properties, key):
        _type = properties[key]['type']
        _result = ''
        def date_valid(x): return x if x is not None else ""
        def num_valid(x): return str(x) if x is not None else ""
        try:
            if _type == 'rich_text' or _type == 'title':
                for i in properties[key][_type]:
                    _result += i['plain_text']

            elif _type == 'multi_select':
                for i in properties[key][_type]:
                    _result += """<span style="color:{color};">{content}</span>""".format(
                        **{'color': i['color'], 'content': i['name']})
            elif _type == 'select':
                i = properties[key][_type]
                _result += """<span style="color:{color};">{content}</span>""".format(
                    **{'color': i['color'], 'content': i['name']})
            elif _type == 'url':
                _result += """<a href="{url}" target="_blank">{text}</a>""".format(
                    **{'url': properties[key][_type], 'text': properties[key][_type]})
            elif _type == 'date':
                _result += """{s} {e}""".format(
                    **{'s': date_valid(properties[key][_type]['start']), 'e': date_valid(properties[key][_type]['end'])})
            elif _type == 'number':
                _result += num_valid(properties[key][_type])
            elif _type == 'files':
                for f in properties[key][_type]:

                    if 'file' in f:
                        url = f['file']['url']
                    elif 'external' in f:
                        url = f['external']['url']

                    img_template = """
                        <img style="width: 200px;" src="{img}"></img>
                    """
                    imgsrc = self.img_url_to_base64(url)
                    clslog.info(imgsrc[0:32]+'...')
                    if imgsrc:
                        _result += img_template.format(
                            **{'img': imgsrc})
                    else:
                        _result += img_template.format(
                            **{'img': url})
            else:
                clslog.warning("Unsupported: {}".format(properties[key]))
        except Exception as e:
            clslog.critical(
                "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))

        return _result

    def block_parse_table(self, client, b):
        database = client.databases.query(b['id'])
        table = str('<table border="0" cellpadding="0" cellspacing="0" style="border-collapse: collapse; color: #3d3b4f; background-color: #fff; border: 1px solid #cfcfcf; box-shadow: 0 0px 6px rgba(0, 0, 0, 0.1); margin-bottom: 20px; font-size:13px;">')
        heads = []
        if len(database['results']):
            table += '<thead>'
            for h in database['results'][0]['properties']:
                heads.append(h)
            for hi in reversed(heads):
                table += """<th height="19" width="107"
                          style="border: 0.5pt solid #cfcfcf; width: 107pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">{h}</th>""".format(**{'h': hi})
            table += '</thead>'
            table += '<tbody>'
            for i in database['results']:
                p = i['properties']
                table += '<tr height="19" style="height:14.0pt;background-color:rgb(253, 233, 217)">'
                for hi in reversed(heads):
                    table += """<td height="19" width="107"
                          style="border: 0.5pt solid #cfcfcf; width: 107pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">{v}</td>""".format(
                        **{'v': self.block_parse_table_cell_properties(p, hi)})
                table += '</tr>'
            table += '</tbody>'
        table += str('</table>')
        return table

    def block_common_types(self, client, b):
        children = str('')

        img_template = """
            <div style="width:100%;border: 1px dashed #ccc;
                vertical-align: middle;text-align: center; 
                box-shadow: 0px 0px 5px 5px rgba(10,10,10,0.2);
                -moz-box-shadow: 0px 0px 5px 5px rgba(10,10,10,0.2);
                -webkit-box-shadow: 0px 0px 5px 5px rgba(10,10,10,0.2);" >
                <img style="width: 80%; margin-top: 50px; margin-bottom: 50px;" src="{img}"></img>
            </div>
        """
        try:
            if b['type'] == "image":
                try:
                    if 'file' in b['image']:
                        url = b['image']['file']['url']
                    elif 'external' in b['image']:
                        url = b['image']['external']['url']

                    imgsrc = self.img_url_to_base64(url)
                    clslog.info(imgsrc[0:32]+'...')
                    if imgsrc:
                        children += img_template.format(
                            **{'img': imgsrc})
                    else:
                        children += img_template.format(
                            **{'img': url})
                except Exception as e:
                    clslog.critical(
                        "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))
                    clslog.info(b)

            elif b['type'] == "paragraph":
                text = str('')
                for t in b['paragraph']['text']:
                    text += t['plain_text']
                children += """<p>{p}</p>""".format(**{'p': text.strip()})

            elif b['type'] == 'heading_1':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<h1>{head}</h1>""".format(
                    **{'head': text.strip()})
            elif b['type'] == 'heading_2':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<h2>{head}</h2>""".format(
                    **{'head': text.strip()})
            elif b['type'] == 'heading_3':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<h3>{head}</h3>""".format(
                    **{'head': text.strip()})
            elif b['type'] == 'heading_4':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<h4>{head}</h4>""".format(
                    **{'head': text.strip()})
            elif b['type'] == 'heading_5':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<h5>{head}</h5>""".format(
                    **{'head': text.strip()})
            elif b['type'] == 'heading_6':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<h6>{head}</h6>""".format(
                    **{'head': text.strip()})
            elif b['type'] == 'code':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<pre style="background-color: f5f5f5"><code>{code}</code></pre>""".format(
                    **{'code': text.strip()})
            elif b['type'] == 'bookmark':
                text = str('')
                for t in b[b['type']]['caption']:
                    text += t['plain_text']
                children += """<a href="{bookmark}" target="_blank">{caption}</a><br/>""".format(
                    **{
                        'bookmark': b[b['type']]['url'],
                        'caption': text.strip()
                    })
            elif b['type'] == 'child_database':
                children += self.block_parse_table(client, b)
            elif b['type'] == 'callout':
                text = str('')
                for t in b[b['type']]['text']:
                    text += t['plain_text']
                children += """<pre style="background-color: f5f5f5"><code>{icon}{code}</code></pre>""".format(
                    **{'code': text.strip(),
                       'icon':  b[b['type']]['icon']['emoji']})
            else:
                clslog.warning("An unsupported type was received")
                clslog.info(b)
        except Exception as e:
            clslog.critical(
                "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))
        return children

    def block_list_children(self, client, block, tag, level=0):
        """Recursive function"""
        bs = client.blocks.children.list(block['id'])
        children = str('')
        if tag == 'ol':
            if level % 4 == 0:
                children = "<" + tag + " type=\"1\">"
            elif level % 4 == 1:
                children = "<" + tag + " type=\"A\">"
            elif level % 4 == 2:
                children = "<" + tag + " type=\"a\">"
            elif level % 4 == 3:
                children = "<" + tag + " type=\"I\">"
            else:
                children = "<" + tag + " type=\"i\">"
        elif tag == 'ul':
            if level % 4 == 0:
                children = "<" + tag + " style=\"list-style-type:\1F44D\">"
            elif level % 4 == 1:
                children = "<" + tag + " style=\"list-style-type:\1F44D\">"
            elif level % 4 == 2:
                children = "<" + tag + " style=\"list-style-type:disc\">"
            elif level % 4 == 3:
                children = "<" + tag + " style=\"list-style-type:\1F44D\">"
            else:
                children = "<" + tag + " style=\"none\">"

        for i in range(len(bs['results'])):
            b = bs['results'][i]

            if b['type'] == 'numbered_list_item' or b['type'] == 'bulleted_list_item':

                plain_text = str('')
                for t in b[b['type']]['text']:
                    plain_text += t['plain_text']
                if b['has_children']:
                    level += 1
                    new_children = str('')
                    if b['type'] == 'bulleted_list_item':
                        new_children += self.block_list_children(
                            client, b, 'ul', level)
                    elif b['type'] == 'numbered_list_item':
                        new_children += self.block_list_children(
                            client, b, 'ol', level)
                    children += """<li>{item}{children}</li>""".format(
                        **{
                            'item':
                            plain_text,
                            'children':
                            new_children
                        })
                else:
                    children += """<li>{item}</li>""".format(
                        **{'item': plain_text})
            else:
                children += self.block_common_types(client, b)

        children += "</" + tag + ">"
        return children

    def render_block_list(self, client, block, tag, level):
        """Recurisve <ol><ul><li> tags management"""
        plain_text = str('')
        for t in block[block['type']]['text']:
            plain_text += t['plain_text']
        if block['has_children']:
            level += 1
            last = """<li>{item}{children}</li>""".format(
                **{
                    'item': plain_text,
                    'children': self.block_list_children(client, block, tag, level)
                })
        else:
            last = """<li>{item}</li>""".format(**{'item': plain_text})
        # clslog.info(last)
        return last

    def img_url_to_base64(self, url):
        try:
            imgtype = os.path.splitext(url)[1].split('?')[0].replace('.', '/')
            response = requests.get(url)
            imgbase64 = base64.b64encode(BytesIO(response.content).read())
            return "data:image{};base64,".format(imgtype) + imgbase64.decode(
                "utf-8")
        except Exception as e:
            clslog.critical(
                "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))
            return None

    def render_block_items(self, client, page):
        page_html_tags = str('')
        p_template = """
            <p style="text-indent: 2em;">{p}</p>
        """
        img_template = """
            <div style="width:auto;border: 1px dashed #ccc;display: table-cell;
                vertical-align: middle;text-align: center; 
                box-shadow: 0px 0px 5px 5px rgba(10,10,10,0.2);
                -moz-box-shadow: 0px 0px 5px 5px rgba(10,10,10,0.2);
                -webkit-box-shadow: 0px 0px 5px 5px rgba(10,10,10,0.2);" >
                <img style="width: 80%;" src="{img}"></img>
            </div>
        """

        page_id = page['id']
        page_blocks = client.blocks.retrieve(page_id)
        page_content = client.blocks.children.list(page_blocks['id'])
        # clslog.info(json.dumps(page_content))
        """Supported blocks:
        paragraph, numbered_list_item, bulleted_list_item
        emoji icons
        """
        bulleted_list = str('')
        numbered_list = str('')
        last_type = None
        for i in range(len(page_content['results'])):
            b = page_content['results'][i]
            # End of <ol>
            if (last_type == 'numbered_list_item'
                    and b['type'] != 'numbered_list_item') or i == len(
                        page_content['results']) - 1:
                page_html_tags += """<ol type="1">{list}</ol>""".format(
                    **{'list': numbered_list})
                numbered_list = str('')
            # End of <ul>
            if (last_type == 'bulleted_list_item'
                    and b['type'] != 'bulleted_list_item') or i == len(
                        page_content['results']) - 1:
                page_html_tags += """<ul style="list-style-type:disc">{list}</ul>""".format(
                    **{'list': bulleted_list})
                bulleted_list = str('')
            if b['type'] == 'numbered_list_item':
                numbered_list += self.render_block_list(client, b, 'ol', 0)
            elif b['type'] == 'bulleted_list_item':
                bulleted_list += self.render_block_list(client, b, 'ul', 0)

            elif b['type'] == 'paragraph':
                text = str('')
                for t in b['paragraph']['text']:
                    text += t['plain_text']
                page_html_tags += p_template.format(**{'p': text.strip()})
            elif b['type'] == 'image' \
                or b['type'] == 'heading_1' \
                or b['type'] == 'heading_2' \
                or b['type'] == 'heading_3' \
                or b['type'] == 'heading_4' \
                or b['type'] == 'heading_5' \
                or b['type'] == 'heading_6' \
                or b['type'] == 'code' \
                    or b['type'] == 'bookmark':
                page_html_tags += self.block_common_types(client, b)
            else:
                clslog.warning("Type:{}".format(b['type']))
                bs = client.blocks.retrieve(b['id'])
                clslog.info(
                    "{} Not supported by current Notion API, more info please visit {}"
                    .format(
                        bs,
                        "https://developers.notion.com/docs/working-with-page-content#modeling-content-as-blocks"
                    ))

            last_type = b['type']
        return page_html_tags

    def render_reading_books(self, client, database):
        for page in database['results']:
            item = page['properties']
            date = datetime.datetime.fromisoformat(
                item[u'收录日期']['date']['start'])
            """Skip useless databases
            """
            if self.belongs(date):
                self._book += self.content_parse_title(item)
                self._book_content += self.render_block_items(client, page)

    def render_study_note(self, client, database):

        for page in database['results']:
            item = page['properties']
            date = datetime.datetime.fromisoformat(
                item[u'收录日期']['date']['start'])
            """Skip useless databases
            """
            if self.belongs(date):
                title_url = item[u'链接']['url']
                self._study_list += """<h1>{title}</h1><br/>""".format(
                    **{'title': self.content_parse_title(item)})
                if title_url:
                    self._study_list += """<a href="{url}" target="_blank"><p>{title}</p></a><br/>""".format(
                        **{
                            'title': "扩展阅读",
                            'url': title_url
                        })

                self._study_list += self.render_block_items(client, page)
        # clslog.info(self._study_list)

    def render_maintarget(self, database):
        for node in database['results']:
            item = node['properties']
            self._main_target += """<li style="list-style-type: demical;">{p}</li>""".format(
                **{'p': item[u'目标']['title'][0]['plain_text']})

    def render_teamtarget(self, database):
        for node in database['results']:
            item = node['properties']
            self._team_target += """<li style="list-style-type: demical;">{p}</li>""".format(
                **{'p': item[u'目标']['title'][0]['plain_text']})

    def render_technology(self, database):
        _template = """
            <li style="list-style-type: upper-roman; ">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'方案输出' or _type == u'技术预研':
                self._technology += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_review(self, database):
        _template = """
            <li style="list-style-type: upper-roman; ">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'代码评审' or _type == u'配置管理':
                self._review += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_maintainance(self, database):
        _template = """
            <li style="list-style-type: upper-roman; ">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:blue;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'项目维护' or _type == u'IT运维':
                self._maintainance += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_patent(self, database):
        _template = """
            <li style="list-style-type: upper-roman;">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'知识产权建设':
                self._patent += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_techissues(self, database):
        _template = """
            <li style="list-style-type: upper-roman; ">
            <span style="font-weight: bold;">{content}</span>&nbsp;
            <span style="color:red;">{problem}</span>&nbsp;
            <span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'项目支撑' or _type == u'技术问题指导':
                self._tech_issues += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'problem': self.content_parse_richtext(item, u'问题'),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_duties(self, database):
        _template = """
            <li style="list-style-type: upper-roman;">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'技术管理' or _type == u'内部支撑':
                self._duties += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_programming_work(self, database):
        _template = """
            <li style="list-style-type: upper-roman;">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'产品开发':
                self._programming_tasks += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_reading_share(self, database):
        _template = """
            <li style="list-style-type: upper-roman;">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """
        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'总结输出' or _type == u'会议记录' or _type == u'复盘分享':
                self._reading_share += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def render_directions(self, database):
        _template = """
            <li style="list-style-type: upper-roman;">
            <span style="font-weight: bold;">{content}</span>
            &nbsp;<span style="color:green;">{solve}</span>
            &nbsp;<span style="color:blue;">{summary}</span></li>
        """

        for node in database['results']:
            item = node['properties']
            _type = self.content_parse_type(item)
            if _type == u'技术分享' or _type == u'好书分享' or _type == u'好文章分享' or _type == u'工作指导':
                self._directions += _template.format(
                    **{
                        'content': self.content_parse_title(item),
                        'summary': self.content_parse_richtext(
                            item, u'评审、复盘、总结'),
                        'solve': self.content_parse_richtext(item, u'解决方法')
                    })

    def parse_week_tasks(self, database, weekdatestr):
        self.render_technology(database)
        self.render_patent(database)
        self.render_review(database)
        self.render_duties(database)
        self.render_techissues(database)
        self.render_maintainance(database)
        self.render_programming_work(database)
        self.render_reading_share(database)
        self.render_directions(database)

    def render_itor_database(self, client, dbid, title, database, force):
        try:
            for t in title:
                # Use BT-Panel timer task to trigger
                if not force:
                    if self.datetime_now.day != 1:
                        clslog.warning(
                            "Month report will not trigger except the day 1")
                        break

                plain_text = t['plain_text'].strip().replace(' → ', '~')
                """Reading list"""
                if plain_text == u"读书如斯":
                    self.render_reading_books(client, database)
                    break
                if plain_text == u"网络万象":
                    self.render_study_note(client, database)
                    break
                if plain_text == u"公司整体目标":
                    self.render_maintarget(database)
                    break
                if plain_text == u"团队技术方向和目标":
                    self.render_teamtarget(database)
                    break
                """Week report parse"""
                plain_text_head = t['plain_text'][0:3]  # Filter for week tasks
                value = re.compile(r'^[0-9][0-9][0-9]$')
                if t['type'] == 'mention':
                    sdate = datetime.datetime.fromisoformat(
                        t['mention']['date']['start'])
                    edate = datetime.datetime.fromisoformat(
                        t['mention']['date']['end'])
                    weekdatestr = """{s}===>{e}""".format(
                        **{
                            's': sdate.strftime("%Y-%m-%d"),
                            'e': edate.strftime("%Y-%m-%d")
                        })
                    """Skip useless databases
                    """
                    if sdate == edate:
                        break
                    if value.match(plain_text_head):
                        if self.week_belongs(sdate, edate):
                            click.secho("解析{}工作任务".format(weekdatestr),
                                        fg='blue')
                            self.parse_week_tasks(database, weekdatestr)
                        else:
                            clslog.info(
                                "Week {} report N/A".format(weekdatestr))
                            break
        except Exception as e:
            clslog.critical(
                "Exception @{}: {}".format(e.__traceback__.tb_lineno, e))
            traceback.print_exc(e)

    def render_html(self, title):
        """Render html form template

        Note that some of the email display methods only support inline-css style,
        This method support inline-css for table headers and rows.

        Args:
            title (str): Title of html and email subject
        """
        clslog.info("Render[Inline-CSS] html for {}".format(title))

        with open(self.wbname + '.html', encoding='utf-8', mode='w') as f:

            html = string.Template(monthreport.mr_template)

            f.write(
                html.safe_substitute({
                    "title": self._title,
                    "book": self._book,
                    "book_content": self._book_content,
                    "study": self._study_list,
                    "main_target": self._main_target,
                    "team_target": self._team_target,
                    "technology": self._technology,
                    "patent": self._patent,
                    "review": self._review,
                    "technology_issues": self._tech_issues,
                    "maintainance": self._maintainance,
                    "duties": self._duties,
                    "programming_work": self._programming_tasks,
                    "reading_share": self._reading_share,
                    "direction": self._directions,
                    "user": self._user,
                    "department": self._department
                }))

    def render_study_html(self, title):
        """Render html form template

        Note that some of the email display methods only support inline-css style,
        This method support inline-css for table headers and rows.

        Args:
            title (str): Title of html and email subject
        """
        clslog.info(
            "Render study content[Inline-CSS] html for {}".format(title))

        html = string.Template(monthreport.mstudy_template)

        return html.safe_substitute({
            "title": title,
            "book": self._book,
            "book_content": self._book_content,
            "study": self._study_list,
            "user": self._user,
            "department": self._department
        })


def cli_week(client, clsconfig, excel, remove, force, send):
    """Generate work report every week

    Args:
        client (object): Notion client instance
        clsconfig (dict): Config instance of .clslq.json
        excel (bool): Support dump excel or not
        remove (bool): Remove dumped files or not
        force (bool): Force generate or not
        send (bool): Send email or not
    """
    for i in client.search()['results']:
        if i['object'] == 'database':
            try:

                database = client.databases.query(i['id'])
                title = i['title']

                for t in title:
                    plain_text = t['plain_text'].strip().replace(' → ', '~')
                    plain_text_head = t['plain_text'][0:3]
                    value = re.compile(r'^[0-9]+[0-9]$')
                    # Handle valid report database only
                    if plain_text_head == 'WRT':
                        break
                    if value.match(plain_text_head) == None:
                        break
                    enddate = datetime.datetime.fromisoformat(
                        t['mention']['date']['end'])
                    startdate = datetime.datetime.fromisoformat(
                        t['mention']['date']['start'])

                    wrp = WeekReport(plain_text)

                    # Use BT-Panel timer task to trigger
                    if not force:
                        if wrp.now.weekday() != 5:  # 0~6 means Monday~Sunday
                            clslog.warning("Today is not Saturday")
                            break
                    if not wrp.week_belongs(startdate, enddate):
                        click.secho(
                            "[{}~{}] today:{}[weekday:{}] gap:{}days Week report expired"
                            .format(startdate.strftime("%Y-%m-%d"),
                                    enddate.strftime("%Y-%m-%d"),
                                    wrp.now.strftime("%Y-%m-%d"),
                                    wrp.now.weekday(),
                                    abs(startdate - wrp.now).days),
                            fg='green')
                        break
                    wtitle = "{}({})".format(clsconfig.get('wr_title_prefix'),
                                             plain_text)

                    if excel:
                        wrp.excel_worksheet_dump(plain_text, database)
                        df = pandas.read_excel(plain_text + '.xlsx',
                                               sheet_name='WR',
                                               header=1)
                    else:
                        df = wrp.pandas_df_fill(database)
                    pandas.set_option('colheader_justify', 'center')
                    df.style.hide_index()  # Hide index col

                    # with open("debug.json", "w") as file:
                    #     file.write(json.dumps(i))

                    wrp.render_html(clsconfig, wtitle, database)
                    if send:
                        wrp.send_email(clsconfig, wtitle)
                    if remove:
                        wrp.remove_files()

            except Exception as e:
                clslog.error(e)
                traceback.print_exc(e)


def cli_month(client, clsconfig, remove, force, send):
    """Generate work report every month

    Args:
        client (object): Notion client instance
        clsconfig (dict): Config instance of .clslq.json
        remove (bool): Remove dumped files or not
        force (bool): Force generate or not
        send (bool): Send email or not
    """
    mrp = MonthReport(clsconfig.get('mr_title_prefix'))
    mrp.init(clsconfig.get('mr_title_prefix'), clsconfig.get('user'),
             clsconfig.get('department'), force)
    # Use BT-Panel timer task to trigger
    if not force:
        if not (mrp.now.day >= 1 and mrp.now.day <= 5):
            clslog.warning(
                "Month report email only sent when mday [1~5] or in force mode"
            )
            return
    """Get week tasks, dump them into month table
    """
    for i in client.search()['results']:
        # Itor all database
        if i['object'] == 'database':
            try:
                database = client.databases.query(i['id'])
                title = i['title']
                mrp.render_itor_database(client, i['id'], title, database,
                                         force)

            except Exception as e:
                clslog.error(e)
                traceback.print_exc(e)
    try:
        mrp.render_html(mrp.mtitle)
        if force:
            study_title = "C(oncept)T(each)R(eview)S(implify)({}{:02})".format(
                mrp.now.year, mrp.now.month)
        else:
            study_title = "C(oncept)T(each)R(eview)S(implify)({}{:02})".format(
                mrp.now.year, mrp.now.month - 1)
        study_email = mrp.render_study_html(study_title)

        if send:
            mrp.send_email(clsconfig, mrp.mtitle)
            mrp.send_study_email(clsconfig, study_title, study_email)
        if remove:
            mrp.remove_files()
    except Exception as e:
        clslog.error(e)
        traceback.print_exc(e)


@click.option('--rtype',
              '-t',
              required=True,
              default='week',
              type=click.Choice(['week', 'month', 'all']),
              help='Choose a type to generate report')
@click.option('--excel',
              '-e',
              flag_value='GenerateExcel',
              default=False,
              help='Generate xlsx excel file or not')
@click.option('--remove',
              '-r',
              flag_value='RemoveFiles',
              help='Remove files or not')
@click.option(
    '--force',
    '-f',
    flag_value='force',
    default=False,
    help='Force generate right now, otherwise limited by valid datetime')
@click.option('--send',
              '-s',
              flag_value='send',
              default=False,
              help='Send email or not')
@click.option('--config',
              '-c',
              type=click.Path(exists=True),
              default='.clslq.json',
              help='CLSLQ config use {} as default.'.format('.clslq.json'))
@click.command(context_settings=dict(
    allow_extra_args=True,
    ignore_unknown_options=True,
),
    help="Notion Report Generator.")
def notion(rtype, config, excel, remove, force, send):

    clsconfig = ClslqConfigUnique(file=config)
    if clsconfig.get('secrets_from') is None:
        click.secho("Make sure notion secret code is valid in .clslq.json",
                    fg='red')
        return

    client = Client(auth=clsconfig.get('secrets_from'),
                    notion_version="2021-08-16")

    click.secho("Report type: {} Notion Accounts:".format(rtype), fg='yellow')
    for u in client.users.list()['results']:
        click.secho("{:8s} {}".format(u['name'], u['id']), fg='green')
    if 'week' == rtype:
        click.secho("Week report generator", fg='green')
        cli_week(client, clsconfig, excel, remove, force, send)
    elif 'month' == rtype:
        click.secho("Month report generator", fg='green')
        cli_month(client, clsconfig, remove, force, send)
    else:
        click.secho("All reports generator", fg='green')
        cli_week(client, clsconfig, excel, remove, force, send)
        cli_month(client, clsconfig, remove, force, send)



wr_template = """

<tbody>
  <tr>
    <td>
      <table align="center" border="0" cellpadding="0" cellspacing="0" 
        style="border-collapse: collapse; background-color: #fff; border: 1px solid #cfcfcf; box-shadow: 0 0px 6px rgba(0, 0, 0, 0.1); margin-bottom: 20px; font-size:13px;">
        <tbody>
          <tr height="34" style="height:34.0pt;vertical-align: middle; text-align: center;padding: 10px; background-color: #f3f9f1; border: none; font-size: 20px; font-weight: bold; border-bottom: 1px solid #e5e5e5;">
            <td style="color: #333; ">$title</td>
          </tr>
          <tr>
            <td style="padding: 10px; border: none;">
              <fieldset style="border: 1px solid #e5e5e5">
                <legend style="color: #114f8e">本周工作情况</legend>
                <div style="padding:5px;">
                  <table border="0" cellpadding="0" cellspacing="0" style="border-collapse: collapse; color: #3d3b4f; background-color: #fff; border: 1px solid #cfcfcf; box-shadow: 0 0px 6px rgba(0, 0, 0, 0.1); margin-bottom: 20px; font-size:13px;">
                    <tbody>
                      <tr height="19" style="height:14.0pt;background-color:rgb(253, 233, 217)">
                        <td height="19" width="107"
                          style="border: 0.5pt solid #cfcfcf; width: 107pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          分类</td>
                        <td height="19" width="284"
                          style="border: 0.5pt solid #cfcfcf; width: 284pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          事项</td>
                        <td height="19" width="84"
                          style="border: 0.5pt solid #cfcfcf; width: 84pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          进度</td>
                        <td height="19" width="279"
                          style="border: 0.5pt solid #cfcfcf; width: 279pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          遇到问题</td>
                        <td height="19" width="381"
                          style="border: 0.5pt solid #cfcfcf; width: 381pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          解决办法</td>
                      </tr>

                      $table

                    </tbody>
                  </table>
                </div>
              </fieldset>
            </td>
          </tr>
          <tr>
            <td style="padding:0px 10px 10px 10px; border: none;">
              <fieldset style="border: 1px solid #e5e5e5">
                <legend style="color: #114f8e">下周工作计划</legend>
                <div style="padding:5px;">
                  <table border="0" cellpadding="0" cellspacing="0" style="border-collapse: collapse; color: #3d3b4f; background-color: #fff; border: 1px solid #cfcfcf; box-shadow: 0 0px 6px rgba(0, 0, 0, 0.1); margin-bottom: 20px; font-size:13px;">
                    <tbody>

                      <tr height="19" style="height:14.0pt;background-color:rgb(253, 233, 217)">
                        <td height="19" width="107"
                          style="border: 0.5pt solid #cfcfcf; width: 107pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          分类</td>
                        <td height="19" width="284"
                          style="border: 0.5pt solid #cfcfcf; width: 284pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          事项</td>
                        <td height="19" width="84"
                          style="border: 0.5pt solid #cfcfcf; width: 84pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          进度</td>
                        <td height="19" width="279"
                          style="border: 0.5pt solid #cfcfcf; width: 279pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          面临问题</td>
                        <td height="19" width="381"
                          style="border: 0.5pt solid #cfcfcf; width: 381pt; height: 14pt; padding-top: 1px; padding-right: 1px; padding-left: 1px; font-size: 11pt; font-family: 宋体; vertical-align: middle; text-align: center;">
                          解决办法</td>
                      </tr>

                      $plan

                    </tbody>
                  </table>
                </div>
              </fieldset>
            </td>
          </tr>
          <tr height="24" style="height:24.0pt;">
            <td style="padding: 10px; background-color: #FFF0D5">
              <span style="font-size: 16px; color: #F1A325">●</span>&nbsp;
              <span>
                <span style="border-bottom: 1px rgb(204, 204, 204); position: relative;">自省、总结、复盘</span>
              </span>
            </td>
          </tr>

          $summarize

        </tbody>
      </table>
    </td>
  </tr>
</tbody>
<!--Footer-->
<div>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">&nbsp;</span></b>
    </p>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">
            <span lang="EN-US"
                style="font-family: Times New Roman, serif; color: rgb(0, 147, 221);">&nbsp;&nbsp;&nbsp;&nbsp;Best
                regards</span>
        </b>
    </p>
    <p
        style="margin: 0px 0cm; font-family: Microsoft YaHei UI, Tahoma; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">
            <span lang="EN-US" style="font-size: 12pt;">
                <font face="黑体">&nbsp;&nbsp;&nbsp;&nbsp;$user</font>
                <font face="Calibri, sans-serif">&nbsp;</font>
            </span>
        </b>
        <b style="font-family: Calibri, sans-serif;">
            <span lang="EN-US" style="font-size: 14pt; font-family: 黑体;">&nbsp;</span>
            <span lang="EN-US" style="font-family: 黑体; font-size: 12px;">$department</span>
        </b>
    </p>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">&nbsp;</span></b>
    </p>
</div>

"""


# -*- encoding: utf-8 -*-
'''
CLSLQ is a python library and command toolsets of Connard

Most of the contents are written in progress of python learning

MIT License Copyright (c) Connard Lee

'''

from .clslq_singleton import clslq_singleton
from .clslq_singleton import SingletonClass
from .clslq_singleton import SingletonMetaclass

from .clslq_config import ClslqConfig
from .clslq_config import ClslqConfigUnique

from .clslq_log import ClslqLogger
from .clslq_md5 import ClslqMd5

from .clslq_sql import *

from .version import CLSLQ_Version

from .clslq_utils import *

__all__ = [
    'SingletonClass', 'SingletonMetaclass', 'clslq_singleton'
    'ClslqConfig', 'ClslqConfigUnique', 'ClslqLogger', 'mkdir_p', 'ClslqMd5',
    'ClslqSql', 'ClslqBaseTable'
]
"""Logger wapper"""
__clslq_log = ClslqLogger()
clslog = __clslq_log.log

__version__ = CLSLQ_Version



# -*- encoding: utf-8 -*-
'''
clslq_config

Created: 2021/08/23 14:26:54

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''


import os
import sys
import json
import configparser
import traceback
from enum import Enum
from xml.etree import ElementTree as ET

from .clslq_singleton import SingletonMetaclass

try:
    '''pyyaml package is required'''
    import yaml
except Exception as e:
    traceback.format_exc()


class ClslqConfigType(Enum):
    """Support types: Json, Yaml, Ini, Xml
    """
    CCJson = 1
    CCYaml = 2
    CCIni = 3
    CCXml = 4


class ClslqConfig(object):
    """ClslqConfig is a config manager

    Aim to support json/xml/ini/yaml parse and dump operations. 

    Object can be easily used ad python dict type.

    Json: object managed as dict

    Ini: object managed as dict

    Xml: ElementTree managed object

    Yaml: object managed as dict

    """
    _filepath = None
    _execpath = None
    _config = None
    _type = ClslqConfigType.CCJson
    '''Only valid as xml config'''
    _tree = None

    def __init__(self, file="config.json") -> None:
        self._filepath = os.path.dirname(os.path.abspath(__file__))
        self._execpath = os.path.dirname(os.path.abspath(sys.argv[0]))
        sys.path.append(self._filepath)
        sys.path.append(self._execpath)
        self.load(file)

    def __repr__(self) -> str:
        return '<ClslqConfig[{} @{}@{}] {}>'.format(self._type, self._filepath,
                                                    self._execpath,
                                                    self._config)

    def _guess_suffix(self, file):
        try:
            suffix = os.path.splitext(file)[-1]
        except:
            pass
        if suffix == '.json':
            return ClslqConfigType.CCJson
        elif suffix == '.yaml':
            return ClslqConfigType.CCYaml
        elif suffix == '.ini':
            return ClslqConfigType.CCIni
        elif suffix == '.xml':
            return ClslqConfigType.CCXml
        else:
            return None

    def get(self, key=None):
        """get config key from parsed contents

        Args:
            key (<str>, optional): key of value or object. Defaults to None.

        Returns:
            None
        """
        try:
            if self._type == ClslqConfigType.CCXml:
                if key:
                    return self._config.find(key)
            elif self._type == ClslqConfigType.CCJson or \
                    self._type == ClslqConfigType.CCIni or \
                    self._type == ClslqConfigType.CCYaml:
                return self._config[key]
            return self._config
        except Exception as e:
            # print("ClslqConfig get '{}' failed[{}]".format(key, e))
            # traceback.print_exc()
            pass

    def load(self, file):
        """load config file

        Args:
            file: relative or absolute path to file

        Raises:
            Exception.args: Exception.args("Not supported")
        """
        self._type = self._guess_suffix(file)
        try:
            if self._type == ClslqConfigType.CCIni:
                self._config = configparser.ConfigParser()
                self._config.read(file)
            elif self._type == ClslqConfigType.CCXml:
                self._tree = ET.parse(file)
                self._config = self._tree.getroot()
            else:
                with open(file, 'r', encoding="utf-8") as fp:
                    if self._type == ClslqConfigType.CCJson:
                        self._config = json.load(fp)
                    elif self._type == ClslqConfigType.CCYaml:
                        fdata = fp.read()
                        self._config = yaml.safe_load(fdata)
                    else:
                        raise Exception.args("Not supported")
        except Exception as e:
            # print(e)
            pass
        finally:
            # print(self._config)
            pass

    def save(self, file):
        """dump config contents to file

        Args:
            file (Path): Where to dump file, path would be relative or absolute

        Raises:
            Exception.args: Exception.args("Not supported")

        Returns:
            False: If config content type is not match <file>'s suffix
        """
        savetype = self._guess_suffix(file)
        if self._type != savetype:
            return False
        try:
            if self._type == ClslqConfigType.CCIni:
                self._config.write(file)
            elif self._type == ClslqConfigType.CCXml:
                self._tree.write(file, encoding='utf-8', xml_declaration=True)
            else:
                with open(file, 'r', encoding="utf-8") as fp:
                    if self._type == ClslqConfigType.CCJson:
                        json.dump(self._config, fp, indent=4)
                    elif self._type == ClslqConfigType.CCYaml:
                        yaml.dump(self._config, file)
                    else:
                        raise Exception.args("Not supported")
        except Exception as e:
            # print(e)
            pass


class ClslqConfigUnique(ClslqConfig, metaclass=SingletonMetaclass):
    """Globally unique ClslqConfig object

    Args:
        metaclass: Defaults to SingletonMetaclass.
    """
    pass



# -*- encoding: utf-8 -*-
'''clslq_sql

Created: 2021/08/23 14:27:37

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''



from sqlalchemy import create_engine
# text SQL statement
from sqlalchemy import *
from sqlalchemy.engine import url
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from contextlib import contextmanager
from sqlalchemy.orm import relationship

# Base Data object
ClslqBaseTable = declarative_base()


class ClslqSql(object):
    """wheel of SQL operations

    :param dburl: sqlalchemy url parameter

        dialect+driver://username:password@host:port/database

        password is URL encoded(import urllib.parse)
       
        sqlite://<nohostname>/<path>, where <path> is relative

        sqlite:///foo.db, means foo.db in current path

        sqlte+pysqlte:///:memory:

        sqlite:////absolute/path/to/foo.db, absolute after '///'

        sqlite:///C:\\path\\to\\foo.db, Windows absolute path

        r'sqlite:///C:\\path\\to\\foo.db', Windows alternative using raw string

        engine = create_engine('sqlite://') Using SQLite :memory:

        postgresql://scott:tiger@localhost:5432/mydatabase

        postgresql+psycopg2://scott:tiger@localhost/mydatabase

        postgresql+pg8000://scott:tiger@localhost/mydatabase

        mysql://scott:tiger@localhost/foo

        mysql+mysqldb://scott:tiger@localhost/foo

        mysql+pymysql://scott:tiger@localhost/foo

        oracle://scott:tiger@127.0.0.1:1521/sidname

        oracle+cx_oracle://scott:tiger@tnsname

        mssql+pyodbc://scott:tiger@mydsn

        mssql+pymssql://scott:tiger@hostname:port/dbname
    """
    __engine = None
    __session = None

    def __init__(self) -> None:
        super().__init__()

    def __repr__(self) -> str:
        return super().__repr__()

    def create(self, dburl):
        """Create engine 
        """
        self.__engine = create_engine(dburl, echo=True)
        self.__session = scoped_session(sessionmaker(bind=self.__engine))

    def init(self, driver, user, passwd, host, port, dbname):
        """Init database

        Args:
            driver (str): pymysql/mysqldb/pyodbc
            user (str): username of database connection
            passwd (str): pasword of database connection
            host (str): database host
            port (str): database port
            dbname (str): database instance name
        """
        self.create(
            url.URL(drivername=driver,
                    username=user,
                    password=passwd,
                    host=host,
                    port=port,
                    database=dbname))

    def create_table(self, table=ClslqBaseTable):
        """
        Create table use MetaData, Column, 
        
        More on https://www.osgeo.cn/sqlalchemy/core/type_basics.html
        """
        try:
            table.metadata.create_all(self.__engine)
        except Exception as e:
            raise

    def insert(self, item=None):
        """Insert an item

        Args:
            item (MetaData, optional): Basic data structure. Defaults to None.

        Raises:
            Exception: Any exception
            e: Any exception
        """
        if not item:
            raise Exception('item is required')
        dbsession = sessionmaker(bind=self.__engine)
        session = dbsession()
        try:
            session.add(item)
            session.commit()
        except Exception as e:
            session.rollback()
            raise e
        finally:
            session.close()

    def delete(self, item=None):
        if not item:
            raise Exception('item is required')
        dbsession = sessionmaker(bind=self.__engine)
        session = dbsession()
        try:
            session.delete(item)
            session.commit()
        except Exception as e:
            session.rollback()
            raise e
        finally:
            session.close()

    def query(self, table=None):
        '''
        How to filter

        result.filter(Table.attr == value)

        How to update

        result.filter(Table.attr == value).update({'attr': 'new_value'})
        filter().delete()
        filter().all()
    
        '''
        if not table:
            raise Exception('table is required')
        session = self.__session()
        try:
            result = session.query(table)
        except Exception as e:
            raise e
        finally:
            session.close()
        return result

    def commit(self):
        session = self.__session()
        session.commit()


mr_template = """

<tbody>
    <tr>
        <td>
            <table border="0" cellpadding="0" cellspacing="0"
                style="border-collapse: collapse; width:100%; background-color: #fff; border: 1px solid #cfcfcf; box-shadow: 0 0px 6px rgba(0, 0, 0, 0.1); margin-bottom: 20px; font-size:13px;">
                <tbody>
                    <tr height="34"
                        style="height:34.0pt;vertical-align: middle; text-align: center;padding: 10px; background-color: #f3f9f1; border: none; font-size: 20px; font-weight: bold; border-bottom: 1px solid #e5e5e5;">
                        <td style="color: #333; ">$title</td>
                    </tr>
                    <tr>
                        <td style="padding: 10px; border: none;">
                            <fieldset style="border: 1px solid #e5e5e5">
                                <legend style="color: #114f8e;font-size: 16pt;">一、读书：$book</legend>
                                <div style="padding:5px;">
                                    <p
                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                        $book_content
                                    </p>
                                </div>
                            </fieldset>
                        </td>
                    </tr>
                    <tr>
                        <td style="padding:0px 10px 10px 10px; border: none;">
                            <fieldset style="border: 1px solid #e5e5e5">
                                <legend style="color: #114f8e;font-size: 16pt;">二、月度工作</legend>
                                <div style="padding:5px;">

                                    <table border="0" cellspacing="0" cellpadding="0"
                                        style="border-collapse:collapse;border:none; width:100%">
                                        <tbody>
                                            <!--表头-->
                                            <tr style="font-weight: bold;">
                                                <td width="44"
                                                    style="width:33.3pt;border:solid windowtext 1.0pt;background:#C2D69B;text-align: center;padding: 10px;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">维度
                                                        </span>
                                                    </p>
                                                </td>
                                                <td width="220" colspan="2"
                                                    style="width:214.7pt;border:solid windowtext 1.0pt;border-left:none;background:#C2D69B;text-align: center;padding: 10px;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">要求</span>
                                                    </p>
                                                </td>
                                                <td width="472"
                                                    style="width:353.65pt;border:solid windowtext 1.0pt;border-left:none;background:#C2D69B;text-align: center;padding: 10px;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">月度进度与结果</span>
                                                    </p>
                                                </td>
                                            </tr>
                                            <!--整体目标-->
                                            <tr>
                                                <td width="44" rowspan="2"
                                                    style="width:33.3pt;border:solid windowtext 1.0pt;border-top:none;text-align: center;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">目标</span>
                                                    </p>
                                                </td>
                                                <td width="420" colspan="2"
                                                    style="width:314.7pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">清楚公司整体目标</span>
                                                    </p>
                                                </td>
                                                <td width="472"
                                                    style="width:353.65pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$main_target</span>
                                                </td>
                                            </tr>
                                            <!--拆解目标-->
                                            <tr>
                                                <td width="420" colspan="2"
                                                    style="width:314.7pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family: 宋体; ">根据公司</span>
                                                        <span
                                                            style="font-family: 宋体; color: red; ">目标清楚职责范围内分解团队技术方向和目标</span>
                                                    </p>
                                                </td>
                                                <td width="472"
                                                    style="width:353.65pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$team_target</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="44" rowspan="6"
                                                    style="width:33.3pt;border:solid windowtext 1.0pt;border-top:none;text-align: center;padding: 10px;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">职责</span>
                                                    </p>
                                                </td>
                                                <td width="48" rowspan="2"
                                                    style="width:35.8pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;text-align: center;padding: 10px;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">技术</span>
                                                    </p>
                                                </td>
                                                <td width="372"
                                                    style="width:278.9pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family: 宋体; color: red; ">公司系统构整体技术架设计</span>
                                                        <span
                                                            style="font-family: 宋体; ">，新技术预言和可行性分析。<br>技术方案的评估、分解、输出。</span>
                                                    </p>
                                                </td>
                                                <td width="472"
                                                    style="width:353.65pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$technology</span>
                                                </td>
                                            </tr>
                                            <tr style="height:12.75pt">
                                                <td width="372"
                                                    style="width:278.9pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt;height:12.75pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        专利输出（上半年 3个，下半年 3个）
                                                    </p>
                                                </td>
                                                <td width="472"
                                                    style="width:353.65pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt;height:12.75pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$patent</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="48" rowspan="4"
                                                    style="width:35.8pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;text-align: center;padding: 10px">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family:宋体;">事
                                                        </span>
                                                    </p>
                                                </td>
                                                <td width="372"
                                                    style="width:278.9pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        组织阶段技术评审对交付代码质量负责
                                                    </p>
                                                </td>
                                                <td width="472"
                                                    style="width:353.65pt;border-top:none;border-left:none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$review</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        技术问题和难题的协助和推进
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$technology_issues</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        运维相关的技术或者资源问题进行辅导
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$maintainance</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        规范软件开发流程，执行制度流程上职责负责的内容
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$duties</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="44" rowspan="2"
                                                    style="width:33.3pt;border:solid windowtext 1.0pt;border-top:none;text-align: center;padding: 10px">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        结果
                                                    </p>
                                                </td>
                                                <td width="48" rowspan="2" style="width:35.8pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            text-align: center;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        事
                                                    </p>
                                                </td>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span
                                                            style="font-family: 宋体; color: red; background-image: initial; background-position: initial; background-size: initial; 
                                                            background-repeat: initial; background-attachment: initial; background-origin: initial; background-clip: initial;">无严重质量问题为前提：
                                                            <br></span>团队项目完成率（&gt;=85%)，=20%
                                                        <br>团队项目完成率（75%&lt;X&lt;85%)，=15%
                                                        <br>团队项目完成率（60%&lt;X&lt;75%)，=10%
                                                        <br>团队项目完成率（&lt;60%)，或者出现严重质量问题=0%
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        见团队实际项目情况
                                                    </p>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        根据项目要求完成编码输出工作
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$programming_work</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="44" rowspan="2"
                                                    style="width:33.3pt;border:solid windowtext 1.0pt;border-top:none;text-align: center;">
                                                    <p
                                                        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        学习
                                                    </p>
                                                </td>
                                                <td width="48"
                                                    style="width:35.8pt;border-top:none;border-left:none;border-bottom:
                                            solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="text-align: justify; margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span lang="EN-US" style="font-family:宋体;">
                                                            <o:p>&nbsp;</o:p>
                                                        </span>
                                                    </p>
                                                </td>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family: 宋体; ">
                                                            读书学到的应用，应用到日常工作和生活中。
                                                            <br>1.</span>月度总结报告呈现。（6%）
                                                        <br>2.月度总结会议中呈现。（2%）
                                                        <br>3.项目例会中呈现。（2%）
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$reading_share</span>
                                                </td>
                                            </tr>
                                            <tr>
                                                <td width="48"
                                                    style="width:35.8pt;border-top:none;border-left:none;border-bottom:
                                            solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="text-align: justify; margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span lang="EN-US" style="font-family:宋体;">
                                                            <o:p>&nbsp;</o:p>
                                                        </span>
                                                    </p>
                                                </td>
                                                <td width="372" style="width:278.9pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <p
                                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                                        <span style="font-family: 宋体; ">指导和影响团队学习。</span>
                                                        <br>1.推荐团队成员优秀
                                                        <span style="font-family: 宋体; color: red; ">书籍</span>
                                                        <span style="font-family: 宋体; ">。（2%）</span>

                                                        <br>2.给予团队
                                                        <span style="font-family: 宋体; color: red; ">培训</span>
                                                        <span style="font-family: 宋体; ">。（5%）</span>

                                                        <br>3.给予团队工作
                                                        <span style="font-family: 宋体; color: red; ">指导</span>
                                                        <span style="font-family: 宋体; ">。（3%）</span>
                                                    </p>
                                                </td>
                                                <td width="472" style="width:353.65pt;border-top:none;border-left:none;
                                            border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
                                            padding:0cm 5.4pt 0cm 5.4pt">
                                                    <span style="font-family: 宋体; font-size: 10.5pt;">$direction</span>
                                                </td>
                                            </tr>
                                        </tbody>
                                    </table>
                                </div>
                            </fieldset>
                        </td>
                    </tr>
                    <tr>
                        <td style="padding: 10px; border: none;">
                            <fieldset style="border: 1px solid #e5e5e5">
                                <legend style="color: #114f8e;font-size: 16pt;">三、学习</legend>
                                <div style="padding:5px;">
                                    <p
                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                        $study
                                    </p>
                                </div>
                            </fieldset>
                        </td>
                    </tr>
                </tbody>
            </table>
        </td>
    </tr>
</tbody>

<!--Footer-->
<div>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">&nbsp;</span></b>
    </p>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">
            <span lang="EN-US"
                style="font-family: Times New Roman, serif; color: rgb(0, 147, 221);">&nbsp;&nbsp;&nbsp;&nbsp;Best
                regards</span>
        </b>
    </p>
    <p
        style="margin: 0px 0cm; font-family: Microsoft YaHei UI, Tahoma; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">
            <span lang="EN-US" style="font-size: 12pt;">
                <font face="黑体">&nbsp;&nbsp;&nbsp;&nbsp;$user</font>
                <font face="Calibri, sans-serif">&nbsp;</font>
            </span>
        </b>
        <b style="font-family: Calibri, sans-serif;">
            <span lang="EN-US" style="font-size: 14pt; font-family: 黑体;">&nbsp;</span>
            <span lang="EN-US" style="font-family: 黑体; font-size: 12px;">$department</span>
        </b>
    </p>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">&nbsp;</span></b>
    </p>
</div>


"""

mstudy_template = """

<tbody>
    <tr>
        <td>
            <table border="0" cellpadding="0" cellspacing="0"
                style="border-collapse: collapse; width:100%; background-color: #fff; border: 1px solid #cfcfcf; box-shadow: 0 0px 6px rgba(0, 0, 0, 0.1); margin-bottom: 20px; font-size:13px;">
                <tbody>
                    <tr height="34"
                        style="height:34.0pt;vertical-align: middle; text-align: center;padding: 10px; background-color: #f3f9f1; border: none; font-size: 20px; font-weight: bold; border-bottom: 1px solid #e5e5e5;">
                        <td style="color: #333; ">$title</td>
                    </tr>
                    <tr>
                        <td style="padding: 10px; border: none;">
                            <fieldset style="border: 1px solid #e5e5e5">
                                <legend style="color: #114f8e;font-size: 16pt;">一、读书：$book</legend>
                                <div style="padding:5px;">
                                    <p
                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                        $book_content
                                    </p>
                                </div>
                            </fieldset>
                        </td>
                    </tr>
                    <tr>
                        <td style="padding: 10px; border: none;">
                            <fieldset style="border: 1px solid #e5e5e5">
                                <legend style="color: #114f8e;font-size: 16pt;">二、学习</legend>
                                <div style="padding:5px;">
                                    <p
                                        style="margin: 0px 0cm; text-align: justify; font-size: 10.5pt; font-family: Calibri, sans-serif;">
                                        $study
                                    </p>
                                </div>
                            </fieldset>
                        </td>
                    </tr>
                </tbody>
            </table>
        </td>
    </tr>
</tbody>

<!--Footer-->
<div>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">&nbsp;</span></b>
    </p>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">
            <span lang="EN-US"
                style="font-family: Times New Roman, serif; color: rgb(0, 147, 221);">&nbsp;&nbsp;&nbsp;&nbsp;Best
                regards</span>
        </b>
    </p>
    <p
        style="margin: 0px 0cm; font-family: Microsoft YaHei UI, Tahoma; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">
            <span lang="EN-US" style="font-size: 12pt;">
                <font face="黑体">&nbsp;&nbsp;&nbsp;&nbsp;$user</font>
                <font face="Calibri, sans-serif">&nbsp;</font>
            </span>
        </b>
        <b style="font-family: Calibri, sans-serif;">
            <span lang="EN-US" style="font-size: 14pt; font-family: 黑体;">&nbsp;</span>
            <span lang="EN-US" style="font-family: 黑体; font-size: 12px;">$department</span>
        </b>
    </p>
    <p
        style="margin: 0px 0cm; font-size: 10.5pt; font-family: Calibri, sans-serif; line-height: 23.8px; widows: 1; text-align: justify; background-color: rgb(225, 245, 255);">
        <b style="font-size: 10.5pt;">&nbsp;</span></b>
    </p>
</div>


"""


# -*- encoding: utf-8 -*-
'''version

Created: 2021/08/23 14:28:01

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''


CLSLQ_Version='1.2.7'
__doc__ = CLSLQ_Version


from setuptools import setup
from setuptools import find_packages
from setuptools import Command
import shutil
import os
from clslq import version
from distutils.sysconfig import get_python_lib

version = version.CLSLQ_Version


def match(list, s):
    if s in list:
        return True
    return False


def rmdir(path):
    removelist = [
        'build', '.pytest_cache', '__pycache__', 'clslq.egg-info', '.eggs',
        'dist'
    ]
    for root, dirs, files in os.walk(path):
        for d in dirs:
            t = os.path.join(root, d)
            father = os.path.basename(root)
            if father == '.git':
                continue
            if os.path.exists(t) and match(removelist, d):
                try:
                    print("delete {}".format(t))
                    shutil.rmtree(t, ignore_errors=True)
                except Exception as e:
                    print(e)

        for f in files:
            t = os.path.join(root, f)
            if os.path.exists(path) and match(removelist, f):
                try:
                    print("delete {}".format(t))
                    os.remove(t)
                except:
                    pass


class CleanCommand(Command):
    description = "distclean"
    user_options = []

    # This method must be implemented
    def initialize_options(self):
        pass

    # This method must be implemented
    def finalize_options(self):
        pass

    def run(self):
        workdir = os.path.dirname(os.path.abspath(__file__))
        print("distclean work root:{}".format(workdir))
        rmdir(workdir)


class DocRunCommand(Command):
    description = "doc run"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system(
            "sphinx-apidoc --maxdepth 5 --separate --force -o source {} setup.py clslq.py"
            .format(os.getcwd()))
        os.system(
            "sphinx-autobuild --host 0.0.0.0 --port 8000 source build/html")


class DocBuildCommand(Command):
    description = "doc run"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system(
            "sphinx-apidoc --maxdepth 5 --separate --force -o source {} setup.py clslq.py"
            .format(os.getcwd()))
        os.system("sphinx-build -D html_theme=bizstyle -D language=zh_CN \
            -D html_logo=logo.png \
            -D html_favicon=favicon.ico \
            -a -b html ./source ./build/zh_CN")
        os.system("sphinx-build -D html_theme=bizstyle -D language=en \
            -D html_logo=logo.png \
            -D html_favicon=favicon.ico \
            -a -b html ./source ./build/en")
        shutil.copytree('source', 'docs')


class DocCreateCommand(Command):
    description = "doc create"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def sphinx_html_sidebars(self):
        with open(os.path.join('source', 'conf.py'), 'a+') as f:
            f.writelines([
                '\r\n', 'html_sidebars = {\n',
                "'**': ['globaltoc.html', 'sourcelink.html', 'searchbox.html'],\n",
                "'using/windows': ['windowssidebar.html', 'searchbox.html'],\n",
                "}\n", "html_theme = 'sphinx_rtd_theme'\n"
            ])

    def run(self):
        shutil.rmtree("source", ignore_errors=True)
        shutil.rmtree(os.path.join("build", 'en'), ignore_errors=True)
        shutil.rmtree(os.path.join("build", 'zh_CN'), ignore_errors=True)
        """sphinx-quickstart create doc tree

        import os
        import sys
        sys.path.insert(0, os.path.abspath('./../'))
        html_theme = 'classic'
        extensions:sphinx.ext.napoleon
        More info @https://www.sphinx-doc.org/en/master/contents.html
        third-party theme https://sphinx-themes.org/
        official theme only clask and bizstyle are recommended

        """
        os.system(
            "sphinx-quickstart --sep {} -p CLSLQ -a Connard.Lee -v {} -r {} -l en \
            --ext-autodoc --ext-intersphinx --ext-doctest --ext-imgmath --ext-todo \
            --extensions sphinx.ext.napoleon \
            --extensions sphinx.ext.autosummary \
            --extensions sphinx.ext.githubpages \
            --extensions sphinx.ext.graphviz\
            --no-makefile --no-batchfile --no-use-make-mod --ext-coverage ".
            format(os.getcwd(), version, version))

        self.sphinx_html_sidebars()
        with open(os.path.join('source', 'conf.py'), 'a+') as f:
            f.writelines([
                '\r\n', 'import os\n', 'import sys\n',
                "sys.path.insert(0, os.path.abspath('./../'))\n"
            ])

        shutil.copyfile('logo.png', os.path.join('source', 'logo.png'))
        shutil.copyfile('favicon.ico', os.path.join('source', 'favicon.ico'))


class PublishCommand(Command):

    description = "Publish a new version to pypi"

    user_options = [
        # The format is (long option, short option, description).
        # python setup.py publish --help
        # python setup.py publish -r/-l
        ("release", 'r', "Publish to pypi.org"),
        ("lovelacelee", 'l', "Publish to pypi.lovelacelee.com"),
    ]

    def initialize_options(self):
        """Set default values for options."""
        self.release = False
        self.lovelacelee = True

    def finalize_options(self):
        """Post-process options."""

        if self.release:
            print("V%s will publish to the https://upload.pypi.org/legacy/" %
                  version)
        if self.lovelacelee:
            print("V%s will publish to the https://pypi.lovelacelee.com/" %
                  version)

    def run(self):
        workdir = os.path.dirname(os.path.abspath(__file__))
        print("distclean work root:{}".format(workdir))
        rmdir(workdir)
        """Run command."""
        os.system("python -m pip install -U setuptools twine wheel")
        os.system("python setup.py sdist bdist_wheel")

        try:
            if self.release:
                os.system("twine upload dist/*")
        except Exception as e:
            print(e)

        try:
            if self.lovelacelee:
                # use .pypirc
                #os.system("twine upload -r lovelacelee dist/*")
                os.system(
                    "twine upload --verbose --username lovelacelee --repository-url https://pypi.lovelacelee.com/ dist/*"
                )
        except Exception as e:
            print(e)

        print("Here is git command tips:")
        print("$ git add .")

        print("$ git commit -m 'publish on version %s'" % version)
        print("$ git tag -a v{} -m 'add tag on {}'".format(version, version))

        print("$ git push")
        print("$ git push origin --tags")


with open('ChangeLog.md', mode='r', encoding='utf-8') as f:
    history = f.read()

setup(
    name="clslq",
    version=version,
    author="Connard.Lee",
    author_email="lovelacelee@gmail.com",
    description="Connard's python library.",
    long_description=history,
    long_description_content_type='text/markdown',
    # Project home
    url="http://git.lovelacelee.com",
    install_requires=[
        'loguru',
        'Click',
        'pipenv',
        'sqlalchemy',
        'notion-client',
        'openpyxl',
        'pandas'
    ],
    platforms=["all"],
    keywords=['clslq', 'clslqutils'],
    # setup.py needs
    setup_requires=['setuptools', 'Click', 'twine', 'sphinx', 'sqlalchemy'],
    requires=['loguru'],
    # python3 setup.py test
    tests_require=[
        'pytest>=3.3.1',
        'pytest-cov>=2.5.1',
        'sqlalchemy',
        'pytest-html',
    ],
    python_requires='>=3',
    # setup_requires or tests_require packages
    # will be written into metadata of *.egg
    dependency_links=[
        #"https://pypi.lovelacelee.com/clslq-1.1.0.tar.gz",
    ],
    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Target users
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        #'Natural Language :: Chinese (Simplified)',

        # Project type
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',

        # Target Python version
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],

    # setuptools.find_packages
    packages=find_packages(exclude=["pytest"]),
    package_dir={'clslq': 'clslq'},
    # Static files: config/service/pictures
    data_files=[
        # root directory such as: c:\python39\
        # Use MANIFEST.in for egg/tar.gz.
        # data_files is required for bdist_wheel

        #('', ['clslq-template.json'])
        #('', ['conf/*.conf']),
        #('/usr/lib/systemd/system/', ['bin/*.service']),
        #('', ['clslq/pip.conf']),
        #('clslq', ['Pipfile']),
    ],
    # Will be packed
    package_data={
        'clslq': ['*.conf', '*.txt', '*.md', '*.html'],
    },
    # Will not be packed
    exclude_package_data={'useless': ['*.in']},
    entry_points={'console_scripts': ["clslq = clslq.cli:main"]},
    cmdclass={
        "distclean": CleanCommand,
        "publish": PublishCommand,
        "docbuild": DocBuildCommand,
        "docrun": DocRunCommand,
        "doccreate": DocCreateCommand,
    })



# -*- encoding: utf-8 -*-
'''cli_clslq_venv

Help to create python virtual environment, pipenv and click module used.

Usage: clslq venv [OPTIONS]

  Python venv manager of CLSLQ implement.

Options:
  -p, --pipconf PATH  Install pip.conf to local system, default use pip.conf.
  -d, --delete        Delete python virtual environment, venv will be created.
  -c, --create        Create python virtual environment, venv will be created.
  -h, --help          Show this message and exit.

'''


import click
import platform
import os
from .clslq_utils import pip_conf_install
from .clslq_utils import pipguess
from .clslq_utils import setenv


@click.option('--create',
              '-c',
              flag_value="venv",
              help='Create python virtual environment, venv will be created.')
@click.option('--delete',
              '-d',
              flag_value="delete",
              help='Delete python virtual environment, venv will be created.')
@click.option('--pipconf',
              '-p',
              type=click.Path(exists=True),
              default=os.path.join(os.path.dirname(__file__), 'pip.conf'),
              help='Install pip.conf to local system, default use {}.'.format(
                  os.path.join(os.path.dirname(__file__), 'pip.conf')))
@click.command(context_settings=dict(
    allow_extra_args=True,
    ignore_unknown_options=True,
),
    help="Python venv manager of CLSLQ implement.")
def venv(create, delete, pipconf):
    setenv(key='PIPENV_TEST_INDEX',
           value='https://pypi.tuna.tsinghua.edu.cn/simple')
    setenv(key='WORKON_HOME', value='venv')
    if pipconf:
        pip_conf_install(pipconf)
    if create:
        click.secho("Create new environment:{}.".format(create), fg='green')
        os.system("pipenv install --three --skip-lock")
        exit()
    if delete:
        click.secho("Delete {}".format(os.path.join(os.getcwd(), 'venv')),
                    fg='green')
        os.system('pipenv --rm')
        exit()

    os.system("pipenv shell")



# -*- encoding: utf-8 -*-
'''clslq_log

Created: 2021/08/23 14:27:03

Contact : lovelacelee@gmail.com

MIT License Copyright (c) 2008~2021 Connard Lee

'''


import sys
import logging
from logging import critical, handlers, info
from .clslq_singleton import SingletonClass
from .clslq_utils import mkdir_p


class ClslqLogger(SingletonClass):
    """Gloable logging wrapper

    This class inherits a module named loguru, ClslqLogger module will use pure logging
    if loguru module not imported correctly.

    """
    _logfile = None
    _filehdr = None
    _loglevel = logging.DEBUG
    _root_logger = logging.RootLogger(logging.DEBUG)
    _fmtstring = '[%(levelname)-8.8s %(asctime)s %(filename)s:%(lineno)d] %(message)s'

    def __init__(self, file=None) -> None:
        super().__init__()
        self._logfile = file
        self._root_logger.handlers.clear()
        '''Console'''
        console = logging.StreamHandler()
        console.setLevel(self._loglevel)
        console.setFormatter(logging.Formatter(self._fmtstring))
        self._root_logger.addHandler(console)
        """
        Instance of TimedRotatingFileHandler
        interval: interval
        backupCount: delete log file more then backupCount files
        when: S-Seconds M-Miniutes H-Hours D-Days W-Weeks(==0 means monday) midnight(00:00)
        """
        if self._logfile:
            mkdir_p(self._logfile)
            self._filehdr = handlers.TimedRotatingFileHandler(self._logfile,
                                                              backupCount=3,
                                                              when='D',
                                                              encoding='utf-8')
            self._filehdr.setFormatter(logging.Formatter(self._fmtstring))
            self._filehdr.setLevel(self._loglevel)
            self._root_logger.addHandler(self._filehdr)

    @property
    def log(self, filename=None):
        """get a logger

        Args:
            filename (Path, optional): Log to file. Defaults to None.

        Returns:
            logger: <loguru.logger> returns if loguru imported correctly, otherwise
            <root_logger> returns
        """
        try:
        
            '''pip install logru'''
            from loguru import logger
            if filename:
                logger.add(
                    "logs/app.log",
                    rotation="2 days",
                    retention="14",
                    format='[{time:YYYY-MM-DD HH:mm:ss} |{level:8.8s}| {file}:{line}]{message}',
                    encoding='utf-8',
                    enqueue=True)
            logger.add(
                sys.stderr,
                format='[<green>{time:YYYY-MM-DD HH:mm:ss}</green> |{level:8.8s}| {file}:<green>{line}</green>]{message}',
                encoding='utf-8',
                enqueue=True,
                colorize=True)
            return logger
        except:
            return ClslqLogger._root_logger

    @property
    def file(self):
        return self._logfile

    @file.setter
    def file(self, path):
        self._logfile = path
